/*
 * File:	interface.cpp
 *
 * Communication Interface for Virtual Classroom Components.
 * interface. This will send and receive messages from the remote
 *
 * Author:	Chirag Vaidya   (cvaidya@cs.pitt.edu)
 *
 * Last changed: March 1, 2004
 *
 */

/*
 * Usage: 	
 *		Step 1: Create your own component which implements Interface
 * 				Component Base.
 *
 *		Step 2:	Replace the ComponentMy with your class name in
 *				main.cpp file's main function.
 *
 *		Step 3:	compile the files using make command on a linux
 *				machine and run the server by typing ./serv
 *
 *		Step 4: Use Virtual Remote which is provided on the web to
 *				send message and get the feedback message.
 *
 */


#include "interface.h"
#include <string.h>
#include <errno.h>
#include <fcntl.h>

KeyValueList::KeyValueList()
{
}

KeyValueList::~KeyValueList()
{
}

/* Look up the value given key, used in getValue() */
int KeyValueList::lookupKey( string strKey )
{
	for( int i = 0; i < keys.size(); i++ )
	{
		string k = (string) keys[i];
		if( strKey == k )
			return i;
	}
	return -1;
}

/* add new (key,value) pair to list */
bool KeyValueList::addPair( string strKey, string strVal )
{
	keys.push_back(strKey);
	values.push_back(strVal);
	return true;
}

/* get the value given key */
string KeyValueList::getValue( string strKey )
{
	int index = lookupKey( strKey );
	if( index == -1 )
		return NULL;
	
	return (string) values[index];
}

/* Show whole list */
string KeyValueList::toString()
{
	string result;

	for( int i = 0; i < keys.size(); i++ )
	{
		result += (string) keys[i] + ":" + (string) values[i] + "\n";
	}
	return result;
}

int KeyValueList::size()
{
	return keys.size();
}

bool KeyValueList::isEmpty()
{
	if( keys.size() )
		return false;
	else
		return true;
}

/* get Key or Value by index */
string KeyValueList::keyAt( int index )
{
	return (string) keys[index];
}

string KeyValueList::valueAt( int index )
{
	return (string) values[index];
}


// Class MsgEncoder functions: Serialize the key value list and send 
// it out to a Stream

const string MsgEncoder::delimiter = "$$$";

MsgEncoder::MsgEncoder( ServerSocket *pSock )
{
	pSocket = pSock;
}

/* Encode the Key Value List into a string and Send it out */
void MsgEncoder::sendMsg( KeyValueList kvList )
{
	if( kvList.isEmpty() )
		return;
	
	string outMsg;

	for( int i = 0; i < kvList.size(); i++ )
	{
		if( outMsg.empty() )
			outMsg = kvList.keyAt(i) + delimiter + kvList.valueAt(i);
		else
			outMsg += delimiter + kvList.keyAt(i) + delimiter + kvList.valueAt(i);
	}
	outMsg += "\n";
	cout << "Sending the message ... " << outMsg;
	(*pSocket) << outMsg;
}

// Class MsgDecoder functions: Get the string from the input stream
// and reconstruct it to a key value list.

const string MsgDecoder::delimiter = "$$$";

MsgDecoder::MsgDecoder( ServerSocket *pSock )
{
	pSocket = pSock;
}

KeyValueList* MsgDecoder::getMsg()
{
	string strMsg;

	(*pSocket) >> strMsg;

	if( strMsg.empty() )
		return NULL;

	KeyValueList *pKvList = new KeyValueList();

	Vector	tokens;
	tokenize( strMsg, tokens, delimiter );
	int i = 0;
	while( i < tokens.size() )
	{
		pKvList->addPair( tokens[i], tokens[i+1] );
		i += 2;
	}
	return pKvList;
}

void tokenize(const string& str,
                      vector<string>& tokens,
                      const string& delimiters = " ")
{
    // Skip delimiters at beginning.
    string::size_type lastPos = str.find_first_not_of(delimiters, 0);
    // Find first "non-delimiter".
    string::size_type pos     = str.find_first_of(delimiters, lastPos);

    while (string::npos != pos || string::npos != lastPos)
    {
		string tmpString;

        // Found a token, add it to the vector. Check if any end of
		// line then strip that
        tmpString = str.substr(lastPos, pos - lastPos);
		if( tmpString[tmpString.size() - 1] == '\n' )
			tmpString = tmpString.substr(0, tmpString.size() - 2);
			//cout << "Ok found a new line in " << tmpString << endl;
        tokens.push_back( tmpString );
        //tokens.push_back(str.substr(lastPos, pos - lastPos));
        // Skip delimiters.  Note the "not_of"
        lastPos = str.find_first_not_of(delimiters, pos);
        // Find next "non-delimiter"
        pos = str.find_first_of(delimiters, lastPos);
    }
}
