/*
 * File:	interface.h
 *
 * Communication Interface for Virtual Classroom Components.
 * interface. This will send and receive messages from the remote
 *
 * Author:	Chirag Vaidya   (cvaidya@cs.pitt.edu)
 *
 * Last changed: March 1, 2004
 *
 */

// Header file for interface.cpp file

#ifndef INTERFACE_CLASS
#define INTERFACE_CLASS

#include "socket.h"
#include <iostream>
#include <unistd.h>
#include <string>
#include <vector>
using namespace std;

typedef vector<string> Vector;

/*
 * Class KeyValueList:
 * 		List of (Key, Value) pair -- the basic format of message
 * 		Keys: MsgID and Description are required for any messages
 * 		MsgID 0-999 is reserved for system use. 
 * 		Your MsgID could start from 1000.
 */

class KeyValueList
{
 	private:
  		Vector keys;
  		Vector values;

		int lookupKey(string strKey);

 	public:
  		KeyValueList();
  		~KeyValueList();
		bool addPair( string strKey, string strVal );
		string getValue( string strKey );
		string toString();
		int size();
		bool isEmpty();
		string keyAt( int index );
		string valueAt( int index );
};

/*
 * 	Class MsgEncoder:
 * 		Serialize the KeyValue List and Send it out to a Stream.
 */

class MsgEncoder
{
	private:
		ServerSocket *pSocket;
		// Default of delimiter in system is $$$
		static const string delimiter;

	public:
		MsgEncoder( ServerSocket *pSock );
		void sendMsg( KeyValueList kvList );
};

/*
 *	Class MsgDecoder:
 * 		Get String from input Stream and reconstruct it to
 *		a Key Value List.
 */
                                                                       
class MsgDecoder {
	private:
		ServerSocket *pSocket;
		static const string delimiter;

	public:
		MsgDecoder( ServerSocket *pSock );
		KeyValueList* getMsg();
};

class ComponentBase
{
	public:
		virtual KeyValueList* processMsg( KeyValueList kvList ) = 0;
};

void tokenize(const string& str, vector<string>& tokens,
                      const string& delimiters);
#endif
