/*
 * File:    main.cpp
 *
 * This file is the main file and starts a new socket and accepts
 * Connections and processes the messages.
 *
 * Author:  Chirag Vaidya   (cvaidya@cs.pitt.edu)
 *
 * Last changed: March 1, 2004
 *
 */

#include "interface.h"
#include "myComponent.h"
#include <string>
#include <iostream>

int main ( int argc, int argv[] )
{
	int	delFlag = 0;

  	cout << "Server running....\n";
  
  	try
    {
      	// Create the socket
      	ServerSocket server ( 7999 );

      	while ( true )
		{
	  		ServerSocket new_sock;
	  		server.accept ( new_sock );
			cout << "Accepted a new client ... \n";

			// You need to create your own component here.
			myComponent mycomp;

	  		try
	    	{
				MsgDecoder mDecoder(&new_sock);
				MsgEncoder mEncoder(&new_sock);

				KeyValueList *pKvInput, kvOutput;

	      		do
				{
					if( delFlag )
					{	// Delete input structure only if the flag is set
						delete pKvInput;
						delFlag = 0;
					}

					pKvInput = mDecoder.getMsg();
					if( pKvInput != NULL )
					{
						cout << "Incoming Message: \t";
						cout << pKvInput->toString() << "\n";

						// Signal pKvInput structure for deletion
						delFlag = 1;	

						KeyValueList *pKvResult;
						pKvResult = mycomp.processMsg(*pKvInput);
						cout << "Outgoing Message: \t";
						cout << pKvResult->toString() << "\n";
						mEncoder.sendMsg( *pKvResult );
						delete pKvResult;
					}
				} while( pKvInput != NULL );
    		}
			catch ( SocketException& ) {}
		}
    }
  	catch ( SocketException& e )
    {
      	cout << "Exception was caught:" << e.description() << "\nExiting.\n";
    }
  	return 0;
}
