/*
 * File:	socket.h
 *
 * This file contains the simple socket interface and the server socket
 * interface. This will send and receive messages from the remote
 *
 * Author:	Chirag Vaidya   (cvaidya@cs.pitt.edu)
 *
 * Last changed: March 1, 2004
 *
 */

// Implementation of the Socket class.

#ifndef Socket_class
#define Socket_class

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#include <string>
#include <arpa/inet.h>

using namespace std;

const int MAXHOSTNAME = 200;
const int MAXCONNECTIONS = 5;
const int MAXRECV = 500;

class Socket
{
 	public:
  		Socket();
  		virtual ~Socket();

  		// Server initialization
  		bool create();
  		bool bind ( const int port );
  		bool listen() const;
  		bool accept ( Socket& ) const;

  		// Data Transimission
  		bool send ( const string ) const;
  		int recv ( string& ) const;

  		void set_non_blocking ( const bool );
  		bool is_valid() const { return m_sock != -1; }

 	private:
  		int m_sock;
  		sockaddr_in m_addr;
};

#endif

// Definition of the ServerSocket class

#ifndef ServerSocket_class
#define ServerSocket_class

class ServerSocket : private Socket
{
 	public:

  		ServerSocket ( int port );
  		ServerSocket (){};
  		virtual ~ServerSocket();

  		const ServerSocket& operator << ( const string& ) const;
  		const ServerSocket& operator >> ( string& ) const;

  		void accept ( ServerSocket& );
};


// SocketException class

class SocketException
{
 	public:
  		SocketException ( string s ) : m_s ( s ) {};
  		~SocketException (){};
  		string description() { return m_s; }

 	private:
  		string m_s;
};

#endif
